
/*
  SDWebServer Partageoir - Example WebServer with SD Card backend for esp8266

  From Copyright (c) 2015 Hristo Gochkov. All rights reserved.
  This file is part of the ESP8266WebServer library for Arduino environment.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  Have a FAT Formatted SD Card connected to the SPI port of the ESP8266
  The web root is the SD Card root folder
  SdFat accept long name
  index.html is the default index (works on subfolders as well)

  upload the contents of SdRoot to the root of the SDcard and access the editor by going to http://esp8266SD.local/edit

  Addon gepeto@SNHack.org ,
   + take parameters from partage.cfg on the SD
   + make a independant AP (softAP)
   + change default ssid/hostname depend on other local wifi if present
*/
#include <Arduino.h>
#include <ESP8266WiFi.h>

#include <WiFiServer.h>
//#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include "./DNSServer.h"                  // Patched lib
#include <SPI.h>
#include <SdFat.h>
//#include <SdFatUtil.h>
//#include <SD.h>
//#include "FS.h"
#include <EEPROM.h>
#include <ArduinoJson.h>

#define DBG_OUTPUT_PORT Serial
#define DEBUG
const byte BUFFER_SIZE = 60;

const byte        DNS_PORT = 53;          // Capture DNS requests on port 53
/* Soft AP network parameters */
IPAddress apIP(192, 168, 4, 1);
IPAddress netMsk(255, 255, 255, 0);

DNSServer         dnsServer;              // Create the DNS object
ESP8266WebServer  server(80);          // HTTP server
/* Don't set this wifi credentials. They are configurated at runtime and stored on EEPROM */
char ssid[32] = "";
char password[32] = "";

/** Should I connect to WLAN asap? */
boolean connect;

/** Last time I tried to connect to WLAN */
long lastConnectTry = 0;

/** Current WLAN status */
int status = WL_IDLE_STATUS;

// si pas de param.txt //
char nomssid[20]  = "Partageoir"; // default ssid partage deviendra partage.local
char passssid[] = ""; // ssid pass
char nomhost[20]  = "partageoir"; // HOST.local partage
int IPC = 4;
int IPD = 1;
char modifpass[] = "admin"; // password to edit
int SDGPIO = 16; // ESP8266-12

SdFat sd;
//SdFile file;
//SdFile dirFile;

static bool hasSD = false;
File uploadFile;
String responseHTML = "Bonjour, pas de carte microSD pour ce PARTAGEOIR... :-(";
String pasdecarteSD = "Bonjour, pas de carte microSD pour ce PARTAGEOIR... :-(";
String detectportalHTML = "Bonjour, ceci est un reseau local";
void returnOK() {
  server.sendHeader("Connection", "close");
  server.sendHeader("Access-Control-Allow-Origin", "*");
  server.send(200, "text/plain", "");
}

void returnFail(String msg) {
  server.sendHeader("Connection", "close");
  server.sendHeader("Access-Control-Allow-Origin", "*");
  server.send(500, "text/plain", msg + "\r\n");
}

bool loadFromSdCard(String path) {
  String dataType = "text/plain";
  if (path.endsWith("/")) path += "index.html";
  if (path.endsWith(".src")) path = path.substring(0, path.lastIndexOf("."));
  else if (path.endsWith(".htm")) dataType = "text/html";
  else if (path.endsWith(".html")) dataType = "text/html";
  else if (path.endsWith(".css")) dataType = "text/css";
  else if (path.endsWith(".js"))  dataType = "application/javascript";
  else if (path.endsWith(".png")) dataType = "image/png";
  else if (path.endsWith(".gif")) dataType = "image/gif";
  else if (path.endsWith(".jpg")) dataType = "image/jpeg";
  else if (path.endsWith(".ico")) dataType = "image/x-icon";
  else if (path.endsWith(".xml")) dataType = "text/xml";
  else if (path.endsWith(".pdf")) dataType = "application/pdf";
  else if (path.endsWith(".zip")) dataType = "application/zip";
  else if (path.endsWith(".gz"))  dataType = "application/x - gzip";
  else if (path.endsWith(".wav")) dataType = "application/octet - stream";
  else if (path.endsWith(".mp3")) dataType = "application/octet - stream";
  else if (path.endsWith(".ogg")) dataType = "application/octet - stream";
  DBG_OUTPUT_PORT.println(path.c_str());
  File dataFile = sd.open(path.c_str());
  if (dataFile.isDirectory()) {
    path += "/index.html";
    dataType = "text/html";
    dataFile = sd.open(path.c_str());
  }

  if (!dataFile)
    return false;

  if (server.hasArg("download")) dataType = "application/octet-stream";

  char buf[1024];
  int siz = dataFile.size();
  DBG_OUTPUT_PORT.println(siz);
  while (siz > 0) {
    size_t len = std::min((int)(sizeof(buf) - 1), siz);
    dataFile.read((uint8_t *)buf, len);
    server.client().write((const char*)buf, len) ;
    siz -= len;
  }
  //if (server.streamFile(dataFile, dataType) != dataFile.size()) {
  //  DBG_OUTPUT_PORT.println("Sent less data than expected!");
  //}
  dataFile.close();
  return true;
}

void handleFileUpload() {
  if (server.uri() != "/edit") return;
  HTTPUpload& upload = server.upload();
  if (upload.status == UPLOAD_FILE_START) {
    if (sd.exists((char *)upload.filename.c_str())) sd.remove((char *)upload.filename.c_str());
    uploadFile = sd.open(upload.filename.c_str(), FILE_WRITE);
    DBG_OUTPUT_PORT.print("Upload: START, filename: "); DBG_OUTPUT_PORT.println(upload.filename);
  } else if (upload.status == UPLOAD_FILE_WRITE) {
    if (uploadFile) uploadFile.write(upload.buf, upload.currentSize);
    DBG_OUTPUT_PORT.print("Upload: WRITE, Bytes: "); DBG_OUTPUT_PORT.println(upload.currentSize);
  } else if (upload.status == UPLOAD_FILE_END) {
    if (uploadFile) uploadFile.close();
    DBG_OUTPUT_PORT.print("Upload: END, Size: "); DBG_OUTPUT_PORT.println(upload.totalSize);
  }
}

void deleteRecursive(String path) {
  File file = sd.open((char *)path.c_str());
  if (!file.isDirectory()) {
    file.close();
    sd.remove((char *)path.c_str());
    return;
  }

  file.rewindDirectory();
  while (true) {
    File entry = file.openNextFile();
    if (!entry) break;
    String entryPath = path + "/" + entry.name();
    if (entry.isDirectory()) {
      entry.close();
      deleteRecursive(entryPath);
    } else {
      entry.close();
      sd.remove((char *)entryPath.c_str());
    }
    yield();
  }

  sd.rmdir((char *)path.c_str());
  file.close();
}

void handleDelete() {
  if (server.args() == 0) return returnFail("BAD ARGS");
  String path = server.arg(0);
  if (path == "/" || !sd.exists((char *)path.c_str())) {
    returnFail("BAD PATH");
    return;
  }
  deleteRecursive(path);
  returnOK();
}

void handleCreate() {
  if (server.args() == 0) return returnFail("BAD ARGS");
  String path = server.arg(0);
  if (path == "/" || sd.exists((char *)path.c_str())) {
    returnFail("BAD PATH");
    return;
  }

  if (path.indexOf('.') > 0) {
    File file = sd.open((char *)path.c_str(), FILE_WRITE);
    if (file) {
      file.write((const char *)0);
      file.close();
    }
  } else {
    sd.mkdir((char *)path.c_str());
  }
  returnOK();
}

void printDirectory() {
  //if (!server.hasArg("dir")) return returnFail("BAD ARGS");
  //String path = server.arg("dir");
    String path = "/";

  if (path != "/" && !sd.exists((char *)path.c_str())) return returnFail("BAD PATH");
  File dir = sd.open((char *)path.c_str());
  path = String();
  if (!dir.isDirectory()) {
    dir.close();
    return returnFail("NOT DIR");
  }
  dir.rewindDirectory();
  server.setContentLength(CONTENT_LENGTH_UNKNOWN);
  server.send(200, "text/json", "");
  WiFiClient client = server.client();

  server.sendContent("[");
  for (int cnt = 0; true; ++cnt) {
    File entry = dir.openNextFile();
    if (!entry)
      break;

    String output;
    if (cnt > 0)
      output = ',';

    output += "{\"type\":\"";
    output += (entry.isDirectory()) ? "dir" : "file";
    output += "\",\"name\":\"";
    output += entry.name();
    output += "\"";
    output += "}";
    server.sendContent(output);
    entry.close();
  }
  server.sendContent("]");
  dir.close();
}
void portailok() {
  server.send(200, "text/plain", "success");
}

void handlepartout() {
  if (hasSD ) {
    if ( loadFromSdCard(server.uri())) {
      DBG_OUTPUT_PORT.println("page ok");
      return; // si ok on a le fichier
    } else {
      DBG_OUTPUT_PORT.print(server.uri());
      DBG_OUTPUT_PORT.println(" retour sur index\n");
      if ( loadFromSdCard("/index.html")) return; // si ok on a le fichier
    }
    String message = "SDCARD Not Detected\n\n";
    message += SDGPIO;
    message += "\nURI: ";
    message += server.uri();
    message += "\nMethod: ";
    message += (server.method() == HTTP_GET) ? "GET" : "POST";
    message += "\nArguments: ";
    message += server.args();
    message += "\n";
    for (uint8_t i = 0; i < server.args(); i++) {
      message += " NAME:" + server.argName(i) + "\n VALUE:" + server.arg(i) + "\n";
    }
    server.send(404, "text/plain", message);
    DBG_OUTPUT_PORT.print(message);

  } else {
    server.send(200, "text/plain", pasdecarteSD);
  }
}
void connectWifi() {
  DBG_OUTPUT_PORT.print("Configuring access point...");
  WiFi.mode(WIFI_AP);
  WiFi.disconnect();
  WiFi.softAPConfig(apIP, apIP, netMsk);
  delay(500);
  //WiFi.disconnect();
  if (strlen(passssid) == 0) {
    WiFi.softAP(nomssid);
    delay(500);
    DBG_OUTPUT_PORT.print("ouverture de ");
    DBG_OUTPUT_PORT.println(nomssid);
  } else {
    WiFi.softAP(nomssid, passssid);
    delay(500);
    DBG_OUTPUT_PORT.print("ouverture  et paswd de ");
    DBG_OUTPUT_PORT.print(nomssid);
    DBG_OUTPUT_PORT.print(" ");
    DBG_OUTPUT_PORT.println(passssid);
  }
  delay(500);
  /* Setup the DNS server redirecting all the domains to the apIP */
  dnsServer.setErrorReplyCode(DNSReplyCode::NoError);
  dnsServer.start(DNS_PORT, "*", apIP);

  DBG_OUTPUT_PORT.print("AP IP address: ");
  DBG_OUTPUT_PORT.println(WiFi.softAPIP());
}
void recup_config_json() {
  // recherche fichier partage.cfg pour y prendre des choses
  File config_file = sd.open("config.json", FILE_READ);
  if (config_file) {
    DBG_OUTPUT_PORT.print("fic config.json");
    size_t size = config_file.size();
    std::unique_ptr<char[]> buf(new char[size]);
    // We don't use String here because ArduinoJson library requires the input
    // buffer to be mutable. If you don't use ArduinoJson, you may as well
    // use configFile.readString instead.
    config_file.readBytes(buf.get(), size);

    StaticJsonBuffer<700> jsonBuffer;
    JsonObject& partagconf = jsonBuffer.parseObject(buf.get());
    if (!partagconf.success()) {
      DBG_OUTPUT_PORT.print("parse pb de lecture de /config.json");
    }
    // SSID PASS HOSTNAME MODIFPASS SSID2 PASS2 HOST2 FILE2 ?
    if ( strlen(partagconf["SSID"]) > 0 ) {
      sprintf(nomssid, "%s", partagconf["SSID"]);
    } else {
      sprintf(nomssid, "%s", "PARTAGEOIR");
    }
    DBG_OUTPUT_PORT.print("ssid ");
    DBG_OUTPUT_PORT.println(nomssid);

    if ( strlen(partagconf["PASSWORD"]) > 0 ) {
      sprintf(passssid, "%s", partagconf["PASSWORD"]);
    } else {
      sprintf(nomssid, "%s", "");
    }
    DBG_OUTPUT_PORT.print("password ");
    DBG_OUTPUT_PORT.println(passssid);

    if ( strlen(partagconf["HOSTNAME"]) > 0 ) {
      sprintf(nomhost, "%s", partagconf["HOSTNAME"]);
    } else {
      sprintf(nomhost, "%s", "partageoir");
    }
    DBG_OUTPUT_PORT.print("hostname ");
    DBG_OUTPUT_PORT.println(nomhost);
  } else {
    sprintf(nomssid, "%s", "PARTAGEOIR");
    sprintf(passssid, "%s", "");
    sprintf(nomhost, "%s", "partageoir");
  }
}
void recup_config() {
  int i , buffer_lenght, line_counter ;
  char buffer[80], *key, *value;
  // recherche fichier partage.cfg pour y prendre des choses
  File config_file = sd.open("partage.cfg", FILE_READ);
  if (config_file) {
    DBG_OUTPUT_PORT.print("fic partage.cfg");
    while (config_file.available() > 0 ) {
      i = 0;
      while ((buffer[i++] = config_file.read()) != '\n') {
        /* Si la ligne dépasse la taille du buffer */
        if (i == BUFFER_SIZE) {
          /* On finit de lire la ligne mais sans stocker les données */
          while (config_file.read() != '\n');
          break; // Et on arrête la lecture de cette ligne
        }
      }
      buffer_lenght = i;

      /* Gestion des lignes trop grande */
      if (i == BUFFER_SIZE) {
        DBG_OUTPUT_PORT.println("Ligne trop longue à la ligne ");
        DBG_OUTPUT_PORT.println(line_counter, DEC);
        DBG_OUTPUT_PORT.println(i);
      }

      /* Finalise la chaine de caractéres ASCIIZ en supprimant le \n au passage */
      buffer[--i] = '\0';

      /* Incrémente le compteur de lignes */
      ++line_counter;

      /* Ignore les lignes vides ou les lignes de commentaires */
      if (buffer[0] == '\0' || buffer[0] == '#') continue;

      /* Cherche l'emplacement de la clef en ignorant les espaces et les tabulations en début de ligne */
      i = 0;
      while (buffer[i] == ' ' || buffer[i] == '\t') {
        if (++i == buffer_lenght) break; // Ignore les lignes contenant uniquement des espaces et/ou des tabulations
      }
      if (i == buffer_lenght) continue; // Gère les lignes contenant uniquement des espaces et/ou des tabulations
      key = &buffer[i];

      /* Cherche l'emplacement du séparateur = en ignorant les espaces et les tabulations âpres la clef */
      while (buffer[i] != '=') {

        /* Ignore les espaces et les tabulations */
        if (buffer[i] == ' ' || buffer[i] == '\t') buffer[i] = '\0';

        if (++i == buffer_lenght) {
          DBG_OUTPUT_PORT.println("Ligne mal forme a la ligne ");
          DBG_OUTPUT_PORT.println(line_counter, DEC);
          break; // Ignore les lignes mal formé
        }
      }
      if (i == buffer_lenght) continue; // Gère les lignes mal formé

      /* Transforme le séparateur = en \0 et continue */
      buffer[i++] = '\0';

      /* Cherche l'emplacement de la valeur en ignorant les espaces et les tabulations âpres le séparateur */
      while (buffer[i] == ' ' || buffer[i] == '\t') {
        if (++i == buffer_lenght) {
          DBG_OUTPUT_PORT.println("Ligne mal forme a la ligne ");
          DBG_OUTPUT_PORT.println(line_counter, DEC);
          break; // Ignore les lignes mal formé
        }
      }
      if (i == buffer_lenght) continue; // Gère les lignes mal formé
      value = &buffer[i];
      DBG_OUTPUT_PORT.print(key);
      DBG_OUTPUT_PORT.print(" : ");
      DBG_OUTPUT_PORT.println(value);
      /* Transforme les données texte en valeur utilisable */
      if (strcmp(key, "SSID") == 0) {
        sprintf(nomssid, "%s", value);
        DBG_OUTPUT_PORT.print("ssid ");
        DBG_OUTPUT_PORT.println(nomssid);
      }
      else if (strcmp(key, "HOSTNAME") == 0) {
        sprintf(nomhost, "%s", value);
        DBG_OUTPUT_PORT.print("nomhost ");
        DBG_OUTPUT_PORT.println(nomhost);
      }
      else if (strcmp(key, "PASSWORD") == 0) {
        sprintf(passssid, "%s", value);
        DBG_OUTPUT_PORT.print("password ");
        DBG_OUTPUT_PORT.println(passssid);
      }
      else if (strcmp(key, "IPC") == 0) {
        IPC = atoi(value);
        DBG_OUTPUT_PORT.println(IPC);
      }
      else if (strcmp(key, "IPD") == 0) {
        IPD = atoi(value);
        DBG_OUTPUT_PORT.println(IPD);
      }
      else if (strcmp(key, "MODIFPASS") == 0) {
        sprintf(modifpass, "%s", value);
        DBG_OUTPUT_PORT.print("modifpass ");
        DBG_OUTPUT_PORT.println(modifpass);
      }
      else { // Default
        DBG_OUTPUT_PORT.print("Clef inconnu ");
        DBG_OUTPUT_PORT.println(key);
      }
      key = "";
      value = "";
    }
    /* Ferme le fichier de configuration */
    config_file.close();
  } else {
    DBG_OUTPUT_PORT.print("pas de fic partage.cfg");
  }

}

void setup(void) {
  DBG_OUTPUT_PORT.begin(9600);
  DBG_OUTPUT_PORT.setDebugOutput(true);
  DBG_OUTPUT_PORT.print("\n");
  if (sd.begin(SDGPIO)) {
    DBG_OUTPUT_PORT.println("SD Card initialized.");
    hasSD = true;
    delay(100);
    recup_config_json();
  } else {
    DBG_OUTPUT_PORT.println("pas de SD ");
    sprintf(nomssid, "%s", "PARTAGEOIR");
    sprintf(passssid, "%s", "");
    sprintf(nomhost, "%s", "partageoir");
    DBG_OUTPUT_PORT.println(nomssid);
    DBG_OUTPUT_PORT.println(passssid);
    DBG_OUTPUT_PORT.println(nomhost);
  }

  /* Setup web pages: root, wifi config pages, SO captive portal detectors and not found. */
  server.on("/", handlepartout);
  //server.on("/success.txt", portailok);
  //server.on("/wifi", handleWifi);
  //server.on("/wifisave", handleWifiSave);
  server.on("/generate_204", handlepartout);  //Android captive portal. Maybe not needed. Might be handled by notFound handler.
  server.on("/fwlink", handlepartout);  //Microsoft captive portal. Maybe not needed. Might be handled by notFound handler.
  //loadCredentials(); // Load WLAN credentials from network
  //connect = strlen(ssid) > 0; // Request WLAN connect if there is a SSID
  //mettre sur la sd server.on("/success.txt", HTTP_POST, [](){ server.send(200, "text/plain", "success"); }, handleFileUpload);
  server.on("/list", HTTP_GET, printDirectory);
  //server.on("/edit", HTTP_PUT, handleCreate);
  //server.on("/edit", HTTP_DELETE, handleDelete);
  //  server.on("/edit", HTTP_POST, [](){ server.send(200, "text/plain", ""); }, handleFileUpload);

  server.on("/edit", HTTP_POST, []() {
    returnOK();
  });
  server.onNotFound ( handlepartout );
  //server.onNotFound([]() {
  //  server.send(200, "text/html", responseHTML);
  //});

  connectWifi();

  server.begin();
  DBG_OUTPUT_PORT.println("HTTP server started");

    // Setup MDNS responder
    if (!MDNS.begin(nomhost)) {
      DBG_OUTPUT_PORT.println("Error setting up MDNS responder!");
    } else {
      DBG_OUTPUT_PORT.println("mDNS responder started");
      // Add service to MDNS-SD
      MDNS.addService("http", "tcp", 80);
    }

}

void loop(void) {
  dnsServer.processNextRequest();
  server.handleClient();
}
